/* Copyright (C) 1998 Alexander Kourakos */
/* Copyright (C) 2021 Arktis Desktop Project, GNU GPLv3 or any later  */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>
#include <sys/time.h>
#include <locale.h>
#include "SDClock.h"
#include "Defaults.h"

options option;
static char program_path[S_L_LENGTH];
static void Version(void);
static void SetOptions(int, char *[]);
static void StringCopyL(char *, const char *);
static void StringCopyM(char *, const char *);
static void StringCopyS(char *, const char *);

int
main(int argc, char *argv[]) {
	int done = FALSE;
	struct timeval tv;
	setlocale(LC_ALL, "");
	StringCopyL(program_path, argv[0]);
	SetOptions(argc, argv);
	CreateWindow(argc, argv);
	while (!done) {
		UpdateClock();
		HandleEvents(&done);
		gettimeofday(&tv, NULL);
		usleep(PERIOD - tv.tv_usec%PERIOD);
	}
	DestroyWindow();
	return EXIT_SUCCESS;
}

static void
Version(void) {
	printf("This is " NAME " " VER "\n");
	printf("Copyright (C) 1999-2000 Alexander Kourakos\n");
	printf("Copyright (C) 2021 Arktis Desktop Project, GNU GPLv3 \n");
	printf("Please see /root/.SDClock/README for detailed information.\n");
}

static void
SetOptions(int ac, char *av[]) {
	int opt_index = 0, o;
	static char *short_opts = "A:a:B:b:C:c:D:dE:eF:G:g:H:hK:k:L:M:NP:p:R:r:S:svw:X:Y:";
	static struct option long_opts[] =
	{
	{"ar", 1, 0, 'a'},
	{"bg", 1, 0, 'B'},
	{"dc", 1, 0, 'D'},
	{"dl", 1, 0, 'L'},
	{"fn", 1, 0, 'F'},
	{"hc", 1, 0, 'H'},
	{"he", 1, 0, 'E'},
	{"help", 0, 0, 'h'},
	{"hl", 1, 0, 'p'},
	{"hw", 1, 0, 'w'},
	{"ml", 1, 0, 'P'},
	{"mm", 1, 0, 'M'},
	{"nd", 0, 0, 'd'},
	{"ne", 0, 0, 'e'},
	{"nn", 0, 0, 'N'},
	{"ns", 0, 0, 's'},
	{"rr", 1, 0, 'R'},
	{"sc", 1, 0, 'S'},
	{"se", 1, 0, 'A'},
	{"sl", 1, 0, 'r'},
	{"sw", 1, 0, 'b'},
	{"sx", 1, 0, 'K'},
	{"sy", 1, 0, 'k'},
	{"version", 0, 0, 'v'},
	{"xa", 1, 0, 'C'},
	{"ya", 1, 0, 'c'},
	{"xd", 1, 0, 'G'},
	{"yd", 1, 0, 'g'},
	{"xx", 1, 0, 'X'},
	{"yy", 1, 0, 'Y'},
	{NULL, 0, 0, 0}
	};
/* Begin by setting the default options (defined in Defaults.h). */
	option.ar = ARC;
	option.bg_pix[0] = '\0';
	option.he = HHEXT;
	option.hl = R_HHL;
	option.hw = R_HMW;
	option.ml = R_MML;
	option.nd = NDATE;
	option.ne = NHEXT;
	option.nn = DIGIC;
	option.ns = NSECS;
	option.se = SSEXT;
	option.sl = SECHL;
	option.sw = SECHW;
	option.sx = X_AXX;
	option.sxy = S_AXX;
	option.sy = Y_AXX;
	option.xa = X_AXX;
	option.xd = XDATE;
	option.xx = HOR_X;
	option.ya = Y_AXX;
	option.yd = YDATE;
	option.yy = VRT_Y;
	StringCopyM(option.dl, LDATE);
	StringCopyM(option.fn, DFONT);
	StringCopyM(option.mm, MMBCB);
	StringCopyM(option.rr, RMBCB);
	StringCopyS(option.dc, DCOLR);
	StringCopyS(option.hc, HHMMC);
	StringCopyS(option.sc, SECHC);

/* Loop through the user-provided options.*/
	while ((o = getopt_long(ac, av, short_opts, long_opts, &opt_index)) != EOF) {
		switch (o) {
			case 'A':
				option.se = atoi(optarg);
				break;
			case 'a':
				option.ar = atoi(optarg);
				break;
			case 'B':
				StringCopyM(option.bg_pix, optarg);
				break;
			case 'b':
				option.sw = atoi(optarg);
				break;
			case 'C':
				option.xa = atoi(optarg);
				break;
			case 'c':
				option.ya = atoi(optarg);
				break;
			case 'D':
				StringCopyS(option.dc, optarg);
				break;
			case 'd':
				option.nd = !NDATE;
				break;
			case 'E':
				option.he = atoi(optarg);
				break;
			case 'e':
				option.ne = !NHEXT;
				break;
			case 'F':
				StringCopyM(option.fn, optarg);
				break;
			case 'G':
				option.xd = atoi(optarg);
				break;
			case 'g':
				option.yd = atoi(optarg);
				break;
			case 'H':
				StringCopyS(option.hc, optarg);
				break;
			case 'h':
				Version();
				exit(EXIT_SUCCESS);
				break;
			case 'K':
				option.sx = atoi(optarg);
				option.sxy = !S_AXX;
				break;
			case 'k':
				option.sy = atoi(optarg);
				option.sxy = !S_AXX;
				break;
			case 'L':
				StringCopyM(option.dl, optarg);
				break;
			case 'M':
				StringCopyM(option.mm, optarg);
				break;
			case 'N':
				option.nn = !DIGIC;
				break;
			case 'P':
				option.ml = atoi(optarg);
				break;
			case 'p':
				option.hl = atoi(optarg);
				break;
			case 'R':
				StringCopyM(option.rr, optarg);
				break;
			case 'r':
				option.sl = atoi(optarg);
				break;
			case 'S':
				StringCopyS(option.sc, optarg);
				break;
			case 's':
				option.ns = !NSECS;
				break;
			case 'v':
				Version();
				exit(EXIT_SUCCESS);
				break;
			case 'w':
				option.hw = atoi(optarg);
				break;
			case 'X':
				option.xx = atoi(optarg);
				break;
			case 'Y':
				option.yy = atoi(optarg);
				break;
			default:
				exit(EXIT_FAILURE);
		}
	}
	if (optind < ac)
	fprintf(stderr, "SDClock: unknown arguments ignored\n");
}

static void
StringCopyL(char *destination, const char *source) {
	strncpy(destination, source, S_L_LENGTH);
	destination[S_L_LENGTH - 1] = '\0';
}

static void
StringCopyM(char *destination, const char *source) {
	strncpy(destination, source, S_M_LENGTH);
	destination[S_M_LENGTH - 1] = '\0';
}

static void
StringCopyS(char *destination, const char *source) {
	strncpy(destination, source, S_S_LENGTH);
	destination[S_S_LENGTH - 1] = '\0';
}

